%global rocm_release 6.0
%global rocm_patch 0

%global somajor   1
%global sominor   2

%define rocmname    rocm-opencl-runtime
%define libname     %mklibname %{rocmname} %{rocm_release}
%define devel_name  %mklibname %{rocmname} -d

%global install_prefix %{_libdir}/rocm

# No need to intall huge rocm-llvm
%global __requires_exclude ^rocm-llvm(.*)

%define subrel 3
Name:           rocm-amd-opencl
Version:        %{rocm_release}.%{rocm_patch}
Release:        %mkrel 0
Summary:        ROCm OpenCL Runtime for AMD GPU
Group:          System/Kernel and hardware
Url:            https://github.com/ROCm-Developer-Tools/clr
License:        MIT
Source0:        https://github.com/ROCm-Developer-Tools/clr/archive/rocm-%{version}.tar.gz#/clr-rocm-%{version}.tar.gz

BuildRequires:  comgr-devel(rocm) = %{version}
BuildRequires:  hsakmt-devel(rocm) = %{version}
BuildRequires:  mesa-common-devel
BuildRequires:  pkgconfig(opengl)
BuildRequires:  pkgconfig(numa)
BuildRequires:  pkgconfig(ocl-icd)
BuildRequires:  pkgconfig(OpenCL-Headers)
BuildRequires:  rocm-cmake = %{version}
BuildRequires:  rocm-llvm-devel = %{version}
BuildRequires:  rocm-runtime-devel = %{version}

Requires:       %{devel_name} = %{version}-%{release}
Requires:       comgr(rocm) = %{version}
Requires:       hsakmt(rocm) = %{version}
Requires:       rocm-core = %{version}
Requires:       rocm-device-libs = %{version}
Requires:       rocm-runtime = %{version}
Requires:       rocminfo = %{version}

Conflicts:      amdgpupro-opencl-orca
Conflicts:      amdgpupro-opencl-pal

Recommends:     rocm-clinfo

# The kernel support only exists for x86_64, aarch64
# However, for aarch64 immintrin.h: error: "This header is only meant to be used on x86 and x64 architecture"
ExclusiveArch:  x86_64


%description
ROCm OpenCL language runtime for AMD GPU, Navi series (RDNA instruction set)
and most of the Vega and Polaris series (GCN instruction set).
Actually, any GPU of GCN architecture supporting OpenCL 2.0 at least might work;
there is no official comprehensive list of supported GPU, unfortunately.
Otherwise, amdgpupro-opencl-pal and amdgpupro-opencl-orca are proprietary alternatives,
for Vega and Polaris series respectively.

Supports offline and in-process/in-memory compilation.


%package -n %{libname}
Summary:        ROCm OpenCL Libraries
Group:          System/Libraries
#For easier update
Provides:       clr(rocm) = %{version}-%{release}

%description -n %{libname}
Libraires for AMD ROCm OpenCL.


%package -n %{devel_name}
Summary:        ROCm OpenCL development package
Group:          Development/Other
#For easier update
Provides:       clr-devel(rocm) = %{version}-%{release}
Requires:       clr(rocm) = %{version}-%{release}
Requires:       pkgconfig(ocl-icd)

%description -n %{devel_name}
The AMD ROCm OpenCL development package.


%package -n rocm-clinfo
Summary:        ROCm OpenCL platform and device tool
Group:          System/Configuration

%description -n rocm-clinfo
A simple ROCm OpenCL application that enumerates all possible platform and
device information.


%prep
%autosetup -p1 -n clr-rocm-%{version}

# Enable experimental pre vega platforms
sed -i 's/\(ROC_ENABLE_PRE_VEGA.*\)false/\1true/' rocclr/utils/flags.hpp

cat > README.%_real_vendor <<EOF
=============================================================
Users have to be added to render and video groups:

   su -c "usermod -a -G render,video \$USER"

and logout/in or reboot
=============================================================
EOF

%build
export ROCM_PATH="%{install_prefix}"
%cmake \
    -Wno-dev \
    -S "." \
    -B build \
    -DCLR_BUILD_OCL=ON

%cmake_build

%install
%cmake_install

install -m644 README.%_real_vendor README.urpmi

cat > rocm.conf <<EOF
%{install_prefix}
%{_libdir}
EOF
install -Dm644 rocm.conf %{buildroot}%{_sysconfdir}/ld.so.conf.d/rocm.conf

mkdir -p %{buildroot}%{_sysconfdir}/OpenCL/vendors
echo '%{install_prefix}/libamdocl64.so' > 'amdocl64.icd'
install -Dm644 amdocl64.icd %{buildroot}%{_sysconfdir}/OpenCL/vendors/amdocl64.icd

#Specific lib folder for ROCm
mkdir -p %{buildroot}%{install_prefix}
mv %{buildroot}%{_libdir}/lib* %{buildroot}%{install_prefix}

#Avoid file conflicts with opencl-headers package:
mkdir -p %{buildroot}%{_includedir}/rocm
mv %{buildroot}%{_includedir}/CL %{buildroot}%{_includedir}/rocm/CL

#Avoid file conflicts with clinfo package:
mv %{buildroot}/%{_bindir}/clinfo %{buildroot}/%{_bindir}/rocm-clinfo

#Avoid duplicate
rm -rf %{buildroot}/%{_usr}/opencl


%post
#apply configuration
echo "Run ldconfig"
ldconfig


%files
%doc README.urpmi README.md CHANGELOG.md

%files -n %{libname}
%license LICENCE
%exclude %{_datadir}/doc/opencl-asan/LICENSE.txt
%exclude %{_datadir}/doc/opencl/LICENSE.txt
%exclude %{_datadir}/doc/rocm-ocl-icd/LICENSE
%{_sysconfdir}/ld.so.conf.d/rocm.conf
%dir %{_sysconfdir}/OpenCL
%dir %{_sysconfdir}/OpenCL/vendors
%{_sysconfdir}/OpenCL/vendors/amdocl64.icd
%dir %{install_prefix}
%{install_prefix}/libamdocl64.so
%{install_prefix}/libOpenCL.so.%{somajor}
%{install_prefix}/libOpenCL.so.%{somajor}.%{sominor}

%files -n %{devel_name}
%dir %{install_prefix}
%{install_prefix}/libcltrace.so
%{install_prefix}/libOpenCL.so
%dir %{_includedir}/rocm
%{_includedir}/rocm/CL{.,*}

%files -n rocm-clinfo
%license LICENCE
%{_bindir}/rocm-clinfo


%changelog
* Thu Dec 28 2023 squidf <squidf> 6.0.0-0.3
- Update to 6.0.0

* Sat Dec 2 2023 squidf <squid-f> 5.7.1-0.4
- Initial Mageia 9 package, upstream version 5.7.1
